##' Function to run DPM part of the model
##' @title Function to run DPM part of the model  
##' @param Beta.obs A matrix of observed effect size, with
##' G rows and K columns , where K is the number of studies.
##' @param Delta A matrix of delta output from \code{MCMCRun}, with
##' G*K rows and T columns , where T is the number of MCMC iterations.
##' @param C.init A hyperparameter
##' @param pi.alpha A hyperparameter
##' @param iteration The number of MCMC chains wish to run  
##' @param seed An initial seed for random number generator.  

##' @return A matrix of clustering output (cluster assignment), 
##' with G rows and I columns, where I is equal to iteration/thin. 

##' @export
##' @examples
##' \dontrun{
##' ## After completing the MCMC step	
##'	data(test.data)
##'	data(test.label)
##' Data.list <- test.data
##' X.list <-  test.label
##' G <- nrow(Data.list[[1]])
##' K <- length(Data.list)
##' libsize <- lapply(Data.list,colSums)
##' es_seq =function (y,X,lib) {
##'  y=array(y)
##'  v1=log((y[which(X==1)]+1)/(lib[which(X==1)]))
##'  v2=log((y[which(X==0)]+1)/(lib[which(X==0)]))
##'  u=mean(v1) - mean(v2)
##'  return(u)
##' }
##' ## Calculate the observed effect size (get directionality)
##' beta.obs<-matrix(rep(0,G*K),nrow=G,ncol=K) 
##' for (g in 1:G){
##'  for (k in 1:K){
##'    lib<-libsize[[k]]
##'    beta.obs[g,k]<-es_seq(y=Data.list[[k]][g,],X=X.list[[k]],lib=lib)
##'  }
##' }
##' ## Only interested in the clustering of the first 300 DE genes
##' beta.obs.de <- beta.obs[1:300,]
##' delta.de <- MCMC.out[['Delta']][1:(300*3),]
##' C.init <- 10 
##' pi.alpha<- 2  
##' thin <- 20
##' ## Run the DPM part  
##' cluster.out <- DPM(Beta.obs = beta.obs.de, Delta = delta.de,iteration = iteration, 
##' thin = thin, C.init = C.init, pi.alpha = pi.alpha,  seed=12345)
##' cluster.top <- cluster.out[,-c(1:(burnin/thin+1))] ## remove the burnin period
##' rownames(cluster.top) <- paste('gene',1:300,sep='')	
##' ## Performs consensus clustering to summarize posterior
##' library(ConsensusClusterPlus)  
##' copy.upper.lower <- function(m) 
##' {
##'  m[lower.tri(m)] <- t(m)[lower.tri(m)]
##'  m
##'}
##' mydist <- function(y) {
##'  diss.mat <- matrix(,nrow=nrow(y),ncol=nrow(y))
##'  rownames(diss.mat) <- colnames(diss.mat) <- rownames(y)
##'  diag(diss.mat) <- 1
##'  for (i in 1:nrow(diss.mat)) {
##'    for (j in i:ncol(diss.mat)){
##'      diss.mat[i,j] <- sum(y[i,]==y[j,])/ncol(y)    
##'    }
##'  }
##'  diss.mat <- copy.upper.lower(diss.mat)
##' return(1-diss.mat)
##' }
##' results = ConsensusClusterPlus(d=t(cluster.top),maxK=6,reps=50,pItem=0.8,pFeature=1,
##'     title="Consensus Clustering Result",clusterAlg="hc",innerLinkage="ward.D2",
##'     finalLinkage="ward.D2", distance="mydist",seed=1262118388.71279,plot="png") 
##' }

DPM <- function(Beta.obs, Delta, iteration, thin, C.init=10, pi.alpha=2, seed=12345) {

## libraries 
  
library(MCMCpack)
library(mvtnorm)
library(BayesLogit)
library(gtools)
library(gdata)
library(msm)
  
  K <- ncol(Beta.obs)
  G <- nrow(Delta)/ K
  
  update.c <- function(z.mat,c.assign,i) {
    
    cluster.pool<-c(unique(c.assign),max(unique(c.assign))+1)
    z.i <- z.mat[i,]
    
    prob.c <- sapply(unique(c.assign), FUN=function(x) {
      cluster.size <- length(which(c.assign==x))
      cluster.ind <- which(c.assign==x)
      if(cluster.size>1) {z.ave.cluster <- apply(z.mat[cluster.ind,],2,mean)} else{
        z.ave.cluster <- z.mat[cluster.ind,]}
      
      ifelse(cluster.size!=0, 
             ( (cluster.size-1) /(G-1+pi.alpha) ) * 
               dmvnorm(x=z.i,mean=(cluster.size/(cluster.size+1))*z.ave.cluster,sigma=diag((cluster.size+2)/(cluster.size+1),K),log=F) , 
             0)},simplify=T )
    
    prob.jump <- ( pi.alpha/(G-1+pi.alpha) ) *
      dmvnorm(x=z.i,mean=rep(0,K),sigma=diag(2,K),log=F)
    
    # prob.c is a vector of length = length(unique(c)) 
    std.prob <- c(prob.c,prob.jump)/(sum(prob.c)+prob.jump) 
    sample(cluster.pool,size=1,prob=std.prob)
  }
  
  rnd = iteration/thin
  ClusterOut <- matrix(0,nrow=G,ncol=rnd+1)
  set.seed(seed)
  ClusterOut[,1] <- sample(1:C.init,size=G,replace=T)  
  sign.vector<-c(apply(Beta.obs,1,sign))   
  
  for (i in 1:rnd ) {    
    start <- proc.time()
    c.assign.old <- ClusterOut[,i]
    prob.vector <- apply(Delta[,(1+thin*(i-1)):(thin*i)],1,mean)
    scale.prob.vector <-  (prob.vector*sign.vector+1)/2  
    z.vec <- qnorm(scale.prob.vector) 
    z.vec[which(z.vec> 3)] <- 4
    z.vec[which(z.vec< -3)] <- -4
    z.mat <- matrix(z.vec,byrow=T,nrow=G,ncol=K)  ## observed data for clustering
    c.assign.cur<-rep(0,G)
    for (g in 1:G) {
      c.assign.cur[g]<-update.c(z.mat=z.mat,c.assign=c.assign.old, i=g)
    }      
    
    end <- proc.time()
    print(end-start)
    print(i)
    print(table(c.assign.cur))
    
    ClusterOut[,i+1] <- c.assign.cur      
    
  }
  
  return(ClusterOut)
  
}
