##' Function to initialize MCMC chain 
##' @title Function to initialize MCMC chain   
##' @param Data.list A list of K elements, where K is the number of studies, 
##' each element is a RNAseq data matrix with G rows and N columns, 
##' where G is number of genes and N is the sample size. 
##' Genes are matched across all studies.
##' @param X.list A list of K elements, each element includes is a phenotypic
##' condition of the corresponding samples, with case=1, control=0  
##' @param lambda.mean A hyperparameter
##' @param lambda.var A hyperparameter
##' @param eta.mean A hyperparameter
##' @param eta.var A hyperparameter
##' @param m.mean A hyperparameter
##' @param m.var A hyperparameter
##' @param seed An initial seed for random number generator.  

##' @return A list of initial values for each parameter

##' @export
##' @examples
##' \dontrun{
##' data(test.data)
##' data(test.label)
##' Data.list <- test.data
##' X.list <- test.label
##' G <- nrow(Data.list[[1]])
##' K <- length(Data.list)
##' lambda.mean<-0
##' lambda.var<- 10
##' eta.mean<- 0
##' eta.var<- 10 
##' m.mean<- 0
##' m.var<- 10
##' init <- Initialize(Data.list=Data.list,X.list=X.list,
##'           lambda.mean=lambda.mean,lambda.var=lambda.var,
##'           eta.mean=eta.mean,eta.var=eta.var,
##'           m.mean=m.mean,m.var=m.var,seed=12345)
##' }

Initialize <- function(Data.list,X.list,
           lambda.mean=0,lambda.var=10,
           eta.mean=0,eta.var=10,
           m.mean=0,m.var=10,seed=12345) {

## libraries 
  
library(MCMCpack)
library(mvtnorm)
library(BayesLogit)
library(gtools)
library(gdata)
library(msm)
  
### Input
# Data.list: a list of K elements, each element is a RNA-seq dataset;
# X.list:  a list of K elements, each element is the phenotypic indicator for the 
# corresponding study;
# The rest are hyperparameters
                         	
  set.seed(seed)
  
  G <- sapply(Data.list,nrow)[1]  # number of genes
  N <- sapply(Data.list,ncol)    # number of samples
  K <- length(Data.list) # number of studies
  
  libsize <- sapply(Data.list,colSums)
  logT <- lapply(libsize,log)
  
  v.alpha <- v.beta <- v.phi <- K+1
  
  #1. pi    
  pi.init<-runif(1,0,1)
  
  #2. delta   
  delta.init<-rep(0,K*G)  
  for (i in 1:G){
    delta.init[((i-1)*K+1):(i*K)]<-as.vector(rbinom(n=K,size=1,prob=pi.init))
  }
    
  #3. sigma2
  sigma2.de.init<- sigma2.non.de.init<- tau2.init <- xi2.init <- rep(1,K)

  #4. grand mean
  lambda.init<-rnorm(G,mean=lambda.mean,sd=sqrt(lambda.var))
  eta.init<-rnorm(G,mean=eta.mean,sd=sqrt(eta.var))
  m.init<-rep(m.mean, G)

  #5. main model
  beta.init<-rep(0,K*G)
  alpha.init<-rep(0,K*G)
  logphi.init<-rep(0,K*G)
  
  for (g in 1:G) {  
    Cov.beta<- diag(sigma2.non.de.init)
    Cov.alpha<- diag(tau2.init)
    Cov.logphi<- diag(xi2.init)
    
    beta.init[((g-1)*K+1):(g*K)]<-as.vector( rmvnorm(n=1,
                                                     mean=delta.init[((g-1)*K+1):(g*K)]*lambda.init[g],
                                                     sigma=Cov.beta ) )
    alpha.init[((g-1)*K+1):(g*K)]<-as.vector( rmvnorm(n=1,
                                                      mean=rep(eta.init[g],K),
                                                      sigma=Cov.alpha  ) )
    logphi.init[((g-1)*K+1):(g*K)]<-as.vector( rmvnorm(n=1,
                                                       mean=rep(m.init[g],K),
                                                       sigma=Cov.logphi ) )
  }  
  
  #6. omega
  
  omega.init <- vector('list',length=K)
  
  for (k in 1:K) {
    omega.init[[k]] <- matrix(,nrow=G,ncol=N[k])
  }
  for (k in 1:K) {
    for (g in 1:G) {
      omega.init[[k]][g,]<-rpg(num=N[k],h=as.numeric(Data.list[[k]][g,])+1/exp(m.mean),z=0)
    }  
  }
  
  init=list(Prob=pi.init,
            Delta=delta.init,
            Sigma2.DE=sigma2.de.init,
            Sigma2.Non.DE=sigma2.non.de.init,
            Tau2=tau2.init,
            Xi2=xi2.init,
            Lambda=lambda.init,
            Eta=eta.init,
            M=m.init,
            Beta=beta.init,
            Alpha=alpha.init,
            LogPhi=logphi.init,
            Omega=omega.init)
  
  print("MCMC chain initialized")
  return(init)
}
