##' Function to run MCMC chain 
##' @title Function to run MCMC chain   
##' @param Data.list A list of K elements, where K is the number of studies, 
##' each element is an RNAseq data matrix with G rows and N 
##' columns, where G is number of genes and N is the sample size.
##' Genes are matched across all studies.
##' @param X.list A list of K elements, each element includes is a phenotypic
##' condition of the corresponding samples, with case=1, control=0.
##' @param Init.value The output from \code{Initalize} 
##' @param Store.value The output from \code{Store} 
##' @param iteration The number of MCMC chains wish to run  
##' @param lambda.mean A hyperparameter
##' @param lambda.var A hyperparameter
##' @param eta.mean A hyperparameter
##' @param eta.var A hyperparameter
##' @param m.mean A hyperparameter
##' @param m.var A hyperparameter
##' @param seed An initial seed for random number generator.  

##' @return A list of MCMC output matrices for each parameter

##' @export
##' @examples
##' \dontrun{
##' MCMC.out <- MCMCRun(Data.list=Data.list,X.list=X.list,Init.value=init,
##'                   Store.value = store,iteration=iteration,
##'                   lambda.mean=lambda.mean, lambda.var=lambda.var, 
##'                    eta.mean=eta.mean, eta.var=eta.var, 
##'                    m.mean=m.mean, m.var=m.var, seed=12345) 
##' }

MCMCRun <- 
function(Data.list,X.list,Init.value,Store.value,iteration,
                    lambda.mean=0,lambda.var=10,eta.mean=0,eta.var=10,
                    m.mean=0,m.var=10,seed=12345) {

## libraries 
  
library(MCMCpack)
library(mvtnorm)
library(BayesLogit)
library(gtools)
library(gdata)
library(msm)
  
### Input
# Data.list: a list of K elements, each element is a RNA-seq dataset;
# X.list:  a list of K elements, each element is the phenotypic indicator for the 
# corresponding study;
# Init.value: a list of initial values;
# Store.value: a list of empty matrices for storing MCMC results;
# The rest are hyperparameters

  G <- sapply(Data.list,nrow)[1]  # number of genes
  N <- sapply(Data.list,ncol)    # number of samples
  K <- length(Data.list) # number of studies
  
  libsize <- lapply(Data.list,colSums)
  logT <- lapply(libsize,log)
  v.alpha <- v.beta <- v.phi <- K+1
  
  #### alpha,beta update function:

  update.alphabeta <- function(y,lib,X.mat,omega,logphi,delta,lambda,eta,sigma2.de,sigma2.non.de,tau2) {
    
    Omega <- diag(omega)
    r <- 1/exp(logphi)
    
    c = c(eta,lambda*delta)
    C = diag(c(tau2,
               ifelse(delta==0,sigma2.non.de,sigma2.de) ) )
    
    V = solve(t(X.mat)%*%Omega%*% X.mat+solve(C)) 
    z = (y-r)/(2*omega) - lib
    m = V%*%(t(X.mat)%*%Omega%*%z + solve(C)%*%c)
    rmvnorm(n=1,mean=m,sigma=V)
  }

  #### logphi update function 
  
  update.logphi<- function(m,Pi,logphi,alpha,beta,X.list,y,lib) {
    
    y.k<-lapply(y,'[',g,,drop=F)
    
    sum.fun<-function(alpha,beta,X,logphi,lib,y){
      sum( dnbinom(y,size=ifelse(is.na(1/exp(logphi)),2,1/exp(logphi)),
                   mu=exp(lib+alpha+beta*X), log=T) )  
    }
    
    old_logp<-mapply(sum.fun,alpha=alpha,
                     beta=beta,X=X.list,logphi=logphi,
                     lib=lib,y=y.k)
    
    old_hyper<-ifelse(dmvnorm(logphi,mean=rep(m,K),sigma=Pi+1e-3*diag(K), log=F)==0,
                      0.5,
                      dmvnorm(logphi,mean=rep(m,K),sigma=Pi+1e-3*diag(K), log=T) )
    
    old_log_like  <- sum(old_logp) + old_hyper
        
    new.logphi <- rep(NA,length(logphi))
     for (i in 1:length(logphi)){
       new.logphi[i]<-rtnorm(1,mean=logphi[i],sd=sqrt(diag(Pi)[i]),lower=-20, upper=10) #
    }
    
    new_logp<-mapply(sum.fun,alpha=alpha,
                     beta=beta,X=X.list,logphi=new.logphi,
                     lib=lib,y=y.k) 
    
    new_hyper<-ifelse(dmvnorm(new.logphi,mean=rep(m,K),sigma=Pi+1e-3*diag(K), log=F)==0,
                      0.5,
                      dmvnorm(new.logphi,mean=rep(m,K),sigma=Pi+1e-3*diag(K), log=T) )
    
    new_log_like  <- sum(new_logp) +new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.logphi)
          else  return(logphi)
        }
        else return(logphi)
      }
      else return(new.logphi)
    }
    else return(logphi)
    
  }

  ### update lambda, eta, m 
  
  update.lambda<-function(beta,lambda,delta,Sigma) {
    
    old_logp<-dmvnorm(beta,mean=lambda*delta,sigma=Sigma+1e-3*diag(K),log=T)
    
    old_hyper<-dnorm(lambda,mean=lambda.mean,sd=sqrt(lambda.var),log=T)
    
    old_log_like  <- old_logp + old_hyper
    
    new.lambda  <- rnorm(1,mean=lambda,sd=sqrt(lambda.var)) # Proposal
    
    new_logp<-dmvnorm(beta,mean=new.lambda*delta,sigma=Sigma+1e-3*diag(K),log=T)
    
    new_hyper<-dnorm(new.lambda,mean=lambda.mean,sd=sqrt(lambda.var),log=T)
    
    new_log_like  <- new_logp + new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.lambda)
          else  return(lambda)
        }
        else return(lambda)
      }
      else return(new.lambda)
    }
    else return(lambda)    
  } 
  
  update.eta<-function(alpha,eta,Lambda) {
    
    old_logp<-dmvnorm(alpha,mean=rep(eta,K),sigma=Lambda+1e-3*diag(K),log=T)
    
    old_hyper<-dnorm(eta,mean=eta.mean,sd=sqrt(eta.var),log=T)
    
    old_log_like  <- old_logp + old_hyper
    
    new.eta  <- rnorm(1,mean=eta,sd=sqrt(eta.var)) # Proposal
    
    new_logp<-dmvnorm(alpha,mean=rep(new.eta,K),sigma=Lambda+1e-3*diag(K),log=T)
    
    new_hyper<-dnorm(new.eta,mean=eta.mean,sd=sqrt(eta.var),log=T)
    
    new_log_like  <- new_logp + new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.eta)
          else  return(eta)
        }
        else return(eta)
      }
      else return(new.eta)
    }
    else return(eta) 
  } 
  
  update.m<-function(logphi,m,Pi) {
    
    old_logp<-dmvnorm(logphi,mean=rep(m,K),sigma=Pi+1e-3*diag(K),log=T)
    
    old_hyper<-dnorm(m,mean=m.mean,sd=sqrt(m.var),log=T)
    
    old_log_like  <- old_logp + old_hyper

    new.m <- rtnorm(1,mean=m, sd=sqrt(m.var),lower = -20, upper=10) #
     
    new_logp<-dmvnorm(logphi,mean=rep(new.m,K),sigma=Pi+1e-3*diag(K),log=T)
    
    new_hyper<-dnorm(new.m,mean=m.mean,sd=sqrt(m.var),log=T)
    
    new_log_like  <- new_logp + new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.m)
          else  return(m)
        }
        else return(m)
      }
      else return(new.m)
    }
    else return(m) 
  } 
 

#### sigma2.beta, sigma2.alpha, sigma2.log.phi update functions
  
  update.sigma2de<-function(beta,lambda,delta){
    rinvgamma(1,shape=sum(delta)/2,
              scale=(1/2)*sum((delta*(beta-lambda)^2) ) + 5  )  
  }
  
  update.sigma2nonde<-function(beta,lambda,delta){
    rinvgamma(1,shape=(G-sum(delta))/2 ,
              scale=(1/2)*sum( (1-delta)*(beta)^2) + 5    )  
  }
  
  update.tau2<-function(alpha,eta){
    rinvgamma(1,shape=G/2,
              scale=(1/2)*sum((alpha-eta)^2)  )
  }
  
  update.xi2 <-function(logphi,m){
    rinvgamma(1,shape=G/2,
              scale=(1/2)*sum((logphi-m)^2)  )
  }

  ### delta_beta update function
  
  update.deltabeta<- function(old.delta,old.beta,new.beta,
                              sigma2.de.i.k,
                              sigma2.non.de.i.k,y,X,lib,
                              alpha,logphi,lambda) {
    
    new.delta<-1-old.delta 
    
    old_logp= sum( dnbinom(y,size=ifelse(is.na(1/exp(logphi)),2,1/exp(logphi)),
                           mu=exp(lib+alpha+old.beta*X), log=T) ) +  
      dnorm(old.beta,mean=ifelse(old.delta==0,0,lambda),
           sd=ifelse(old.delta==0,sqrt(sigma2.non.de.i.k),sqrt(sigma2.de.i.k)),log=T) 
    
    new_logp= sum( dnbinom(y,size=ifelse(is.na(1/exp(logphi)),2,1/exp(logphi)),
                           mu=exp(lib+alpha+new.beta*X), log=T) ) +  
      dnorm(new.beta,mean=ifelse(new.delta==0,0,lambda),
               sd=ifelse(new.delta==0,sqrt(sigma2.non.de.i.k),sqrt(sigma2.de.i.k)),log=T) 
    
    old_log_like<-old_logp
    new_log_like<-new_logp
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          
          if(new_log_like - old_log_like > log(runif(1)))
            return(c(new.delta,new.beta)) 
          else
            return(c(old.delta,old.beta)) 
        }
        else return (c(old.delta,old.beta))
      }
      else return (c(new.delta,new.beta))
    }
    else 
      return (c(old.delta,old.beta))
    
  }    
  
  ## list of library and count: for block update in the last step
  lib.g<-rep(logT,G)
  X.g<-rep(X.list,G)
  y.k.g<-vector('list',K*G)
  for (g in 1:G){
    for (k in 1:K){
      y.k.g[[(g-1)*K+k]]<-Data.list[[k]][g,]
    }
  }

  ## calculate the observed ES
  
  es.seq =function (y,X,lib) {
    y=array(y)
    v1=log((y[which(X==1)]+1)/(lib[which(X==1)]))
    v2=log((y[which(X==0)]+1)/(lib[which(X==0)]))
    u=mean(v1-v2)
    return(u)
  }
  
  beta.obs<-matrix(rep(0,G*K),nrow=G,ncol=K) 
  for (g in 1:G){
    for (k in 1:K){
      lib<-libsize[[k]]
      beta.obs[g,k]<-es.seq(y=Data.list[[k]][g,],X=X.list[[k]],lib=lib)
    }
  }

## initialize the chain
  
  pi <- Init.value[["Prob"]]
  delta <- Store.value[["Delta"]][,1] <- Init.value[["Delta"]]
  sigma2.de <- Init.value[["Sigma2.DE"]]
  sigma2.non.de <- Init.value[["Sigma2.Non.DE"]]
  tau2 <- Init.value[["Tau2"]]
  xi2 <- Init.value[["Xi2"]]
  lambda <- Init.value[["Lambda"]]
  eta <- Init.value[["Eta"]]
  m <- Init.value[["M"]]
  beta <- Store.value[["Beta"]][,1] <- Init.value[["Beta"]]
  alpha <- Init.value[["Alpha"]]
  logphi <- Init.value[["LogPhi"]]
  omega <- Init.value[["Omega"]]

  ### Iteration part
  for (iter in 2:iteration) {   
    
    set.seed(seed+iter)
    
    startTime <- proc.time()

    ### update alpha+beta,logphi,lambda,eta,m 
    
    alpha.cur=rep(0,K*G)
    beta.cur=rep(0,K*G)
    logphi.cur=rep(0,K*G)
    lambda.cur=rep(0,G)
    eta.cur=rep(0,G)
    m.cur= rep(0,G)
    
    Sigma.de<- diag(sigma2.de)
    Sigma.non.de<- diag(sigma2.non.de)
    Lambda <- diag(tau2)
    Pi <- diag(xi2)
     
    omega.cur  <- omega

    for (g in 1:G) {

      delta.g<-delta[((g-1)*K+1):(g*K)]
      beta.g<- beta[((g-1)*K+1):(g*K)] 
      alpha.g<- alpha[((g-1)*K+1):(g*K)] 
      logphi.g<-logphi[((g-1)*K+1):(g*K)]
      lambda.g<-lambda[g]
      eta.g<- eta[g] 
      m.g<-m[g]
      
    for (k in 1:K) {  	
      
      X.mat <- cbind(rep(1,N[[k]]),X.list[[k]])
      
      alphabeta <- update.alphabeta(y=Data.list[[k]][g,],lib=logT[[k]],X.mat=X.mat,omega=omega[[k]][g,],logphi=logphi.g[k],delta=delta.g[k],lambda=lambda.g,eta=eta.g,sigma2.de=sigma2.de[k],sigma2.non.de=sigma2.non.de[k],tau2=tau2[k])  
           if(abs(alphabeta[2])<=10 & abs(alphabeta[1]) <= 30 ) {
          	     alpha.cur[(g-1)*K+k] <- alphabeta[1]
        	     beta.cur[(g-1)*K+k] <- alphabeta[2]
               } else{   
                  alpha.cur[(g-1)*K+k] <- alpha.g[k]
                  beta.cur[(g-1)*K+k] <- beta.g[k]
               } 

        omega.cur[[k]][g,] <- rpg(num=N[[k]],h=Data.list[[k]][g,]+1/exp(logphi.g[k]),z=c( X.mat%*%c(alpha.cur[(g-1)*K+k],beta.cur[(g-1)*K+k]))+ logT[[k]] )
        
      }
      
     logphi.cur[((g-1)*K+1):(g*K)]<-update.logphi(m=m.g,Pi=Pi,logphi=logphi.g,alpha=alpha.cur[((g-1)*K+1):(g*K)],beta=beta.cur[((g-1)*K+1):(g*K)],X.list=X.list,y=Data.list,lib=logT)         
  
           if(sum(delta.g)==0) {

         lambda.cur[g] <- rnorm(1,0,2)
#         lambda.cur[g] <- rnorm(1,lambda.mean,sqrt(lambda.var))
#        lambda.cur[g]<-update.lambda(beta=beta.cur[((g-1)*K+1):(g*K)] ,lambda=lambda.g,delta=delta.g,Sigma=Sigma.non.de)	
        
      } else  {     

        lambda.cur[g]<-update.lambda(beta=beta.cur[((g-1)*K+1):(g*K)],lambda=lambda.g,delta=delta.g,Sigma=Sigma.de)	

      }

       eta.cur[g]<-update.eta(alpha=alpha.cur[((g-1)*K+1):(g*K)] ,eta=eta.g,Lambda=Lambda)
       m.cur[g]<-update.m(logphi=logphi.cur[((g-1)*K+1):(g*K)],m=m.g,Pi=Pi)
    
   } ## end of g in 1:G

    beta <- beta.cur
    alpha <-alpha.cur
    logphi <-logphi.cur      
    lambda <-lambda.cur  
    eta <-eta.cur
    m <-m.cur
    omega <- omega.cur   
   
    # update sigma2.beta, sigma2.alpha, sigma2.log.phi
    
    sigma2.de.cur<-rep(0,K)
    sigma2.non.de.cur<-rep(0,K)
    tau2.cur<-rep(0,K)
    xi2.cur<-rep(0,K)

    for (k in 1:K) {
      
      sigma2.de.cur[k]<-update.sigma2de(
        beta=beta.cur[seq(k,(G-1)*K+k,by=K)],lambda=lambda,delta=delta[seq(k,(G-1)*K+k,by=K)])
      
      sigma2.non.de.cur[k]<-update.sigma2nonde(
        beta=beta.cur[seq(k,(G-1)*K+k,by=K)],lambda=lambda,delta=delta[seq(k,(G-1)*K+k,by=K)])
      
      tau2.cur[k]<-update.tau2(
        alpha=alpha[seq(k,(G-1)*K+k,by=K)],eta=eta)
      
      xi2.cur[k]<-update.xi2(
        logphi=logphi[seq(k,(G-1)*K+k,by=K)],m=m)
    }

    sigma2.de <- sigma2.de.cur
    sigma2.non.de <- sigma2.non.de.cur
    tau2 <- tau2.cur
    xi2 <- xi2.cur

    ## block update of delta and beta
    
    
    deltabeta.cur<-unlist(mapply(update.deltabeta,
                                 old.delta=delta,old.beta=beta.cur,
                                 new.beta=beta.cur,sigma2.de.i.k =sigma2.de,
                                 sigma2.non.de.i.k=sigma2.non.de,
                                 y=y.k.g,X=X.g,lib=lib.g,
                                 alpha=alpha,logphi=logphi,lambda=rep(lambda,each=K)
    ))

    delta.cur<-deltabeta.cur[1,]
    beta.cur<-deltabeta.cur[2,]
    
    delta <- Store.value[['Delta']][,iter]<- delta.cur
    beta <- Store.value[['Beta']][,iter]<- beta.cur   
    
    endTime <- proc.time()
    
    print(iter)
    print(endTime-startTime) 
    
 } ## end of iteration

    print("MCMC is complete")
    return(Store.value) 

} ## end of MCMCRun function
