##' Function to run parallel MCMC chain 
##' @title Function to run parallel MCMC chain   
##' @param Data.list A list of K elements, where K is the number of studies, 
##' each element is an RNAseq data matrix with G rows and N 
##' columns, where G is number of genes and N is the sample size.
##' Genes are matched across all studies.
##' @param X.list A list of K elements, each element includes is a phenotypic
##' condition of the corresponding samples, with case=1, control=0.
##' @param iteration The number of MCMC chains wish to run  
##' @param chunks The number of data chunks to split into (= number of CPUs called in snowfall) 
##' @param lambda.mean A hyperparameter
##' @param lambda.var A hyperparameter
##' @param eta.mean A hyperparameter
##' @param eta.var A hyperparameter
##' @param m.mean A hyperparameter
##' @param m.var A hyperparameter
##' @param seed An initial seed for random number generator.  

##' @return A list of three components:
##' \itemize{
##' \item{Delta}{ A output matrix of DE indicator estimate delta (Reordered according to gene.name).}
##' \item{Beta}{ A output matrix of effect size estimate beta (Reordered according to gene.name).}
##' \item{Gene.Name}{ Reordered gene names according to random split of original dataset; if the original dataset has no gene names, we will simply use gene(RowNumber) to denote the gene name, e.g. gene1 refers to the first gene.}
##' }

##' @export
##' @examples
##' \dontrun{
##' data(test.data)
##' data(test.label)
##' Data.list <- test.data
##' X.list <- test.label
##' G <- nrow(Data.list[[1]])
##' K <- length(Data.list)
##' lambda.mean<-0
##' lambda.var<- 10
##' eta.mean<- 0
##' eta.var<- 10 
##' m.mean<- 0
##' m.var<- 10
##' iteration <- 2000	
##' burnin <- 500
##' chunks <- 2	
##' MCMC.out <- parMCMC(Data.list=Data.list,X.list=X.list,iteration=iteration,
##'                    chunks = chunks,
##'                   lambda.mean=lambda.mean, lambda.var=lambda.var, 
##'                    eta.mean=eta.mean, eta.var=eta.var, 
##'                    m.mean=m.mean, m.var=m.var, seed=12345) 
##' delta <- MCMC.out[['Delta']]
##' gene.name <- MCMC.out[['Gene.Name']]
##' delta.mean <- apply(delta[,-c(1:burnin)],1,mean)
##' delta.mat <- matrix(delta.mean,nrow=G,ncol=K,byrow=T)
##' rownames(delta.mat) <- gene.name
##' ### Don't forget to turn the output back to the original gene order as in Data.list:
##' delta.mat.order <- delta.mat[rownames(Data.list[[1]]),]
##' }

parMCMC <- function(Data.list,X.list,iteration,chunks,
                    lambda.mean=0,lambda.var=10,
                    eta.mean=0, eta.var=10,
                    m.mean=0, m.var=10, seed=12345) {
	
library(MCMCpack)
library(mvtnorm)
library(BayesLogit)
library(gtools)
library(gdata)
library(msm)
library(snowfall)
library(cvTools)
library(plyr)

  G <- sapply(Data.list,nrow)[1]  # number of genes
  N <- sapply(Data.list,ncol)    # number of samples
  K <- length(Data.list) # number of studies

 for (k in 1:K) {
   if(is.null(rownames(Data.list[[k]]))) {
     rownames(Data.list[[k]]) <- paste("gene",1:G,sep="") 	
  } 
} 
  
  libsize <- lapply(Data.list,colSums)
  logT <- lapply(libsize,log)

if(chunks <= 1) stop("Chunks has to be a number greater than 1")

sfInit(parallel=T,cpus=chunks,type="SOCK")
set.seed(seed)
	
cv.out <- cvFolds(G,K=chunks)
Data.sub <- vector('list',length=chunks)

for (i in 1:chunks) {
   Data.sub[[i]] <- lapply(1:K,FUN=function(x) Data.list[[x]][which(cv.out$which==i),]) 
 }

## ordered gene name
gene.name <- unlist(sapply(1:chunks,FUN=function(x) rownames(Data.sub[[x]][[1]]),simplify=F))
	
Wrapper <- function(data.step, Data.sub) {

Data <- Data.sub[[data.step]]	
init <- Initialize(Data.list=Data,X.list=X.list,
           lambda.mean=lambda.mean,lambda.var=lambda.var,
           eta.mean=eta.mean,eta.var=eta.var,
           m.mean=m.mean,m.var=m.var,seed=seed)
store <-Store(Data.list=Data, iteration=iteration)            	
MCMC.out <- MCMCRun(Data.list=Data,X.list=X.list,Init.value=init,
                   Store.value = store,iteration=iteration,
                   lambda.mean=lambda.mean, lambda.var=lambda.var, 
                    eta.mean=eta.mean, eta.var=eta.var, 
                    m.mean=m.mean, m.var=m.var, seed=seed) 		
return(MCMC.out)   
                    
}	

#sfExport("Initialize") 
#sfExport("Store") 
#sfExport("MCMCRun") 
sfExport("Wrapper") 
sfExport("Data.sub")
sfExport("X.list")
sfExport("lambda.mean")
sfExport("lambda.var")
sfExport("eta.mean")
sfExport("eta.var")
sfExport("m.mean")
sfExport("m.var")
sfExport("iteration")		
sfExport("seed")	
	
time.start <- proc.time()
result<-sfLapply(1:chunks, function(xxx) Wrapper(xxx, Data.sub = Data.sub) ) 
time.end <- proc.time()
print(time.end-time.start)
print("Parallel MCMC is complete")
sfStop()

retrive.delta <- function(x) {
	x[['Delta']]
}
delta <- rbind.fill.matrix(lapply(result,retrive.delta))

retrive.beta <- function(x) {
	x[['Beta']]
}
beta <- rbind.fill.matrix(lapply(result,retrive.beta))

out <- list(delta,beta,gene.name)
names(out) <- c("Delta","Beta","Gene.Name")
return(out)
}